<?php

/**
 * Server 1C class.
 *
 * @category   apps
 * @package    server-1c
 * @subpackage controllers
 * @author     High Tech Lider ltd. <htlead@htlead.kz>
 * @copyright  2018 High Tech Lider
 * @license    http://www.gnu.org/copyleft/gpl.html GNU General Public License version 3 or later
 * @link       https://github.com/htlead/clear_os_1c_server
 */

///////////////////////////////////////////////////////////////////////////////
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// N A M E S P A C E
///////////////////////////////////////////////////////////////////////////////

namespace clearos\apps\server_1c;

///////////////////////////////////////////////////////////////////////////////
// B O O T S T R A P
///////////////////////////////////////////////////////////////////////////////

$bootstrap = getenv('CLEAROS_BOOTSTRAP') ? getenv('CLEAROS_BOOTSTRAP') : '/usr/clearos/framework/shared';
require_once $bootstrap . '/bootstrap.php';

///////////////////////////////////////////////////////////////////////////////
// T R A N S L A T I O N S
///////////////////////////////////////////////////////////////////////////////

clearos_load_language('1c_server');

///////////////////////////////////////////////////////////////////////////////
// D E P E N D E N C I E S
///////////////////////////////////////////////////////////////////////////////

// Classes
//--------

use \clearos\apps\base\File as File;
use \clearos\apps\base\Folder as Folder;
use \clearos\apps\base\Daemon as Daemon;
use \clearos\apps\base\Engine as Engine;
use \clearos\apps\base\Shell as Shell;
use \clearos\apps\network\Network_Utils as Network_Utils;

clearos_load_library('base/File');
clearos_load_library('base/Folder');
clearos_load_library('base/Daemon');
clearos_load_library('base/Engine');
clearos_load_library('base/Shell');
clearos_load_library('network/Network_Utils');

// Exceptions
//-----------

use \clearos\apps\base\Validation_Exception as Validation_Exception;

clearos_load_library('base/Validation_Exception');

///////////////////////////////////////////////////////////////////////////////
// C L A S S
///////////////////////////////////////////////////////////////////////////////

/**
 * Server 1C webapps controller.
 *
 * @category   apps
 * @package    server-1c
 * @subpackage controllers
 * @author     High Tech Lider ltd. <htlead@htlead.kz>
 * @copyright  2018 High Tech Lider
 * @license    http://www.gnu.org/copyleft/gpl.html GNU General Public License version 3 or later
 * @link       https://github.com/htlead/clear_os_1c_server
 */

class Postgres extends Engine
{
    ///////////////////////////////////////////////////////////////////////////////
    // C O N S T A N T S
    ///////////////////////////////////////////////////////////////////////////////
    
    const PG_DATA           = '/var/lib/pgsql/9.6/data/';
    const PG_PATH           = '/usr/pgsql-9.6/bin/';
    const FILE_CONFIG       = '/etc/clearos/server_1c.conf';
    const PG_BACKUP         = '/var/clearos/server_1c/backup';
        
    private $file_config = null;
    
    ///////////////////////////////////////////////////////////////////////////////
    // M E T H O D S
    ///////////////////////////////////////////////////////////////////////////////

    /**
     * Server 1C constructor.
     */

    public function __construct()
    {
        clearos_profile(__METHOD__, __LINE__);        
    }
    
    ///////////////////////////////////////////
    ///////////COMMON CONFIG METHODS /////////
    
    public function load_config() {
    	clearos_profile(__METHOD__, __LINE__);
    	if(! is_null($this->file_config)) {
    	    return $this->file_config;
    	}
    	
    	$section = null;
    	$config  = array();
    	
    	$file = new File(Postgres::FILE_CONFIG, true);
    	if(!$file->exists()) {
		$file->create('root', 'root', '0644');
	}

    	$lines=$file->get_contents_as_array();
	
    	foreach($lines as $line) {
    	
    		$line = trim($line);
    	        if(strlen($line) == 0 || $line[0] == '#') {
    		    continue;
    		}
		$line=preg_replace('/\s+/',' ',$line);
		if(preg_match("/\[([^\]]+)\]/", $line, $m)) {
			$section = $m[1];
			$config[$section] = array();
			continue;
		}
		$line = explode("=", $line, 2);
		$config[$section][trim($line[0])] = trim($line[1]);
    	}

    	$this->file_config = $config;
    	return $config;
    } 
    
    public function save_config(){
	$file = new File(Postgres::FILE_CONFIG, true);
	$config_file=array();
	foreach($this->file_config as $section => $data) {
		$config_file[] = "[$section]\r\n";
		foreach ($data as $key => $value){
		    $config_file[] = "$key = $value\r\n";
		}    
	}
	if($file->exists()) {
		$file->delete();
	}
	
	$file->create('root', 'root', '0644');
	$file->add_lines(implode('', $config_file));
    }   
    
    public function get_config_section($section){
	clearos_profile(__METHOD__, __LINE__);
	$this->load_config();	
	$out=$this->file_config[$section];
	return $out;
    }
        
    function pg_install() {
    	clearos_profile(__METHOD__, __LINE__);
	
	$result='';
	
	$shell = new Shell();
	$options = array();
	$options['validate_exit_code']=FALSE;
	
	try{
	    
	    $args = sprintf('install  %s','webconfig-php-pgsql');
    	    $exitcode = $shell->execute('yum' , $args, TRUE, $options);
    	    $result.="</BR></BR>".implode("</BR>",$shell->get_output());
	    
            $args = sprintf('-ivh %s','http://1c.postgrespro.ru/keys/postgrespro-1c-centos96.noarch.rpm');
    	    $exitcode = $shell->execute('rpm' , $args, TRUE, $options);
    	    $result.="</BR></BR>".implode("</BR>",$shell->get_output());

    	    $args = sprintf('makecache');
	
    	    $exitcode = $shell->execute('yum' , $args, TRUE, $options);
    	    $result.="</BR></BR>".implode("</BR>",$shell->get_output());
    	    
    	    $args = sprintf('install -y %s','postgresql-pro-1c-9.6');
	
    	    $exitcode = $shell->execute('yum' , $args, TRUE, $options);
    	    $result.="</BR></BR>".implode("</BR>",$shell->get_output());
        }catch(Exception $e)
        {
    	    $result.=$e;
        }
        
        return $result;
    }
    
     function pg_initdb()
    {
	clearos_profile(__METHOD__, __LINE__);
	
	$result='';
	
	$shell = new Shell();
	$options = array();
	$options['validate_exit_code']=FALSE;
	
	try{
	$args = sprintf('postgresql-9.6 initdb');

        $exitcode = $shell->execute('service' , $args, TRUE, $options);
        $result.="</BR></BR>".implode("</BR>",$shell->get_output());
        

	}catch(Exceprion $e)
	{
	    $result.=$e;
	}
	
        return $result;
    }
    
    function get_pg_initdb_status()
    {
	clearos_profile(__METHOD__, __LINE__);
	$folder = new Folder(Postgres::PG_DATA , true);
	if($folder->exists()){
	    return true;
	}else{
	    return false;
	}
    }
    
    function pg_status() 
    {
	clearos_profile(__METHOD__, __LINE__);
	$psql = new Daemon('postmaster');
	return $psql->get_running_state();
    }
    
    function pg_start()
    {
	clearos_profile(__METHOD__, __LINE__);
	
	$message='';
	
	$shell = new Shell();
	$options = array();
	$options['validate_exit_code']=FALSE;
	
	try{
	$args = sprintf('postgresql-9.6 start');

        $exitcode = $shell->execute('service' , $args, TRUE, $options);
        $message.="</BR></BR>".implode("</BR>",$shell->get_output());        

	}catch(Exceprion $e)
	{
	    $message.=$e;
	}
	$result=array();
        $result['exitcode']  = $exitcode;
        $result['message']   = $message;
	
        return $result;
    }
    
    function pg_stop()
    {
	clearos_profile(__METHOD__, __LINE__);
	
	$message='';
	
	$shell = new Shell();
	$options = array();
	$options['validate_exit_code']=FALSE;
	
	try{
	$args = sprintf('postgresql-9.6 stop');

        $exitcode = $shell->execute('service' , $args, TRUE, $options);
        $message.="</BR></BR>".implode("</BR>",$shell->get_output());        

	}catch(Exceprion $e)
	{
	    $message.=$e;
	}
	
	$result=array();
        $result['exitcode']  = $exitcode;
        $result['message']   = $message;
	
        return $result;
    }
    
    function connection_add($host, $port, $user, $pass)
    {
	clearos_profile(__METHOD__, __LINE__);
	$this->load_config();	
        $this->file_config['PostgreSQL']['host']    = $host;
        $this->file_config['PostgreSQL']['port']    = $port;
        $this->file_config['PostgreSQL']['user']    = $user;
        $this->file_config['PostgreSQL']['password']= $pass;
        $this->save_config();
    }
    
    function get_db_list()
    {
	clearos_profile(__METHOD__, __LINE__);
	
	$result=array();
	$result['exitcode'] = 0;
	
	$this->load_config();
	
	if(isset($this->file_config['PostgreSQL']['host']))    {$host=$this->file_config['PostgreSQL']['host'];}else{$host='127.0.0.1';}
	if(isset($this->file_config['PostgreSQL']['port']))    {$port=$this->file_config['PostgreSQL']['port'];}else{$port='5432';}
	if(isset($this->file_config['PostgreSQL']['user']))    {$user=$this->file_config['PostgreSQL']['user'];}else{$user='postgres';}
	if(isset($this->file_config['PostgreSQL']['password'])){$pass=$this->file_config['PostgreSQL']['password'];}else{$pass='';}
	
	$connection_string='host='.$host.' port='.$port.' user='.$user.' password='.$pass;
	$db = pg_connect($connection_string);
	if (!$db) {
	  $result['exitcode'] = 1;
	  $connection_string  = str_replace('password='.$pass,'password=***',$connection_string);
	  $result['message'] = lang('server_1c_pg_connetcion_not_esteblished').'<BR>'.$connection_string;
	}
	$query = "SELECT d.datname as name, pg_catalog.pg_get_userbyid(d.datdba) as owner, pg_catalog.pg_encoding_to_char(d.encoding) as encoding,".
		"  d.datcollate as collate, pg_size_pretty(pg_database_size(d.datname)) as db_size FROM pg_catalog.pg_database d ORDER BY 1";
	                                   
	$pg_result = pg_query($query); 
	
	if (!$pg_result) {
	  $result['exitcode'] = 1;
	  $connection_string  = str_replace('password='.$pass,'password=***',$connection_string);
	  $result['message']  = lang('server_1c_pg_connetcion_not_esteblished').'<BR>'.$connection_string;
	}else {
	    while ($database = pg_fetch_array($pg_result)) {
	    	$result['databases'][] = $database;
	    }
	}
        
        return $result;
    }
    
    function set_password($password)
    {
	clearos_profile(__METHOD__, __LINE__);
	
	$result=array();
	$result['exitcode'] = 0;
	
	$this->load_config();
	
	if(isset($this->file_config['PostgreSQL']['host']))    {$host=$this->file_config['PostgreSQL']['host'];}else{$host='127.0.0.1';}
	if(isset($this->file_config['PostgreSQL']['port']))    {$port=$this->file_config['PostgreSQL']['port'];}else{$port='5432';}
	if(isset($this->file_config['PostgreSQL']['user']))    {$user=$this->file_config['PostgreSQL']['user'];}else{$user='postgres';}
	if(isset($this->file_config['PostgreSQL']['password'])){$pass=$this->file_config['PostgreSQL']['password'];}else{$pass='';}
	
	$connection_string='host='.$host.' port='.$port.' user='.$user.' password='.$pass;
	$db = pg_connect($connection_string);
	if (!$db) {
	  $result['exitcode'] = 1;
	  $connection_string  = str_replace('password='.$pass,'password=***',$connection_string);
	  $result['message'] = lang('server_1c_pg_connetcion_not_esteblished').'<BR>'.$connection_string;
	}
	$query = "ALTER USER postgres WITH PASSWORD '".$password."';";
	
	echo $query;                                   
	$pg_result = pg_query($query); 
	
	if (!$pg_result) {
	  $result['exitcode'] = 1;
	  $connection_string  = str_replace('password='.$pass,'password=***',$connection_string);
	  $result['message']  = lang('server_1c_pg_connetcion_not_esteblished').'<BR>'.$connection_string;
	}else {
	    while ($database = pg_fetch_array($pg_result)) {
	    	$result['databases'][] = $database;
	    }
	}
        
        return $result;
    }
    
    function get_access_list()
    {
	clearos_profile(__METHOD__, __LINE__);
    	$config  = array();
    	
    	$file = new File(Postgres::PG_DATA.'pg_hba.conf', true);
    	if($file->exists()) {

    	    $lines=$file->get_contents_as_array();
	
    	    foreach($lines as $line) {
    	
    		$line = trim($line);
    	        if(strlen($line) == 0 || trim($line[0]) == '#') {
    		    continue;
    		}
		$line=preg_replace('/\s+/',' ',$line);
		$line = explode(" ", $line, 5);
		$params["TYPE"]     = trim($line[0]);
		$params["DATABASE"] = trim($line[1]);
		$params["USER"]     = trim($line[2]);
		if(trim($line[0])=='local'){
		    $params["ADDRESS"] = "";
		    $params["METHOD"]  = trim($line[3]);
		}else{
		    $params["ADDRESS"]  = trim($line[3]);
		    $params["METHOD"]   = trim($line[4]);
		}
		$config[]           = $params;
    	    }
    	}
    	
    	return $config;
    }
    
    public function set_access_list($access_list){
	$file        = new File(Postgres::PG_DATA.'pg_hba.conf', true);
	$access_file = array();
	foreach($access_list as $access) {
	    $access_file[] = $access['TYPE'].'    '. $access['DATABASE'].'    '.$access['USER'].'    '.$access['ADDRESS'].'    '.$access['METHOD']."\r\n";
	}
	
	if($file->exists()) {
	    $file->delete();
	}
	
	$file->create('root', 'root', '0644');
	$file->add_lines(implode('', $access_file));
    } 

    function update_access_list($params, $newtype, $newdatabase, $newuser, $newmethod, $newaddress)
    {
	clearos_profile(__METHOD__, __LINE__);

	$exitcode = 1;
	$message  = lang('1c_server_pg_access_not_found');
	$access_list = $this->get_access_list();
	$access_list_new  = array();
	foreach($access_list as $access)
	{
	    if(($access['TYPE']==$params->TYPE)&&($access['DATABASE']==$params->DATABASE)&&($access['USER']==$params->USER)
					      &&($access['METHOD']==$params->METHOD)&&($access['ADDRESS']==$params->ADDRESS)){
		$access_new             = array();
		$access_new['TYPE']     = $newtype;
		$access_new['DATABASE'] = $newdatabase;
		$access_new['USER']     = $newuser;
		$access_new['METHOD']   = $newmethod;
		$access_new['ADDRESS']  = $newaddress;
		$access_list_new[]      = $access_new;
		$exitcode=0;
	    }else{
		
		$access_list_new[]=$access;
	    }
	}

	if($exitcode==0){$this->set_access_list($access_list_new); return;}
	
	$result=array();
        $result['exitcode']  = 1;
        $result['message']   = $message;
        
        return $result;
    }
    
    function get_pg_config()
    {
	clearos_profile(__METHOD__, __LINE__);
    	$config  = array();
    	
    	$file = new File(Postgres::PG_DATA.'----.conf', true);
    	if(!$file->exists()) {

    	    $lines=$file->get_contents_as_array();
	
    	    foreach($lines as $line) {
    	
    		$line = trim($line);
    	        if(strlen($line) == 0 || trim($line[0]) == '#') {
    		    continue;
    		}
		$line=preg_replace('/\s+/',' ',$line);
		$line = explode("=", $line, 2);
		$config[trim($line[0])] = trim($line[1]);
    	    }

    	    return $config;
    	}
    }
    
    function pg_backup($dbname)
    {
	clearos_profile(__METHOD__, __LINE__);
	
	$message='';
	
	$shell = new Shell();
	$options = array();
	$options['validate_exit_code']=FALSE;
	$options['log']='pg_back-up.txt';
	
	try{
	    $set      = $this->get_config_section('PostgreSQL');
	    $args     = sprintf('-h %s -p %s -w -U %s -d %s -F c -b -v -f %s',$set['host'],$set['port'], $set['user'], $dbname, Postgres::PG_BACKUP.'/'.$dbname.date("m-d-y-H-i-s").'.tar');
	    echo  $args;
	    $exitcode = $shell->execute('export PGPASSWORD='.$set['password'], '', TRUE, $options);
	    $exitcode = $shell->execute(Postgres::PG_PATH.'/pg_dump', $args, TRUE, $options);
	    $exitcode = $shell->execute('export PGPASSWORD=','',TRUE,$options);
	    $message .= "</BR></BR>".implode("</BR>",$shell->get_output());
	}catch(Exceprion $e)
	{
	    echo $e->message;
	    $message.=$e->message;
	}
	
	$result=array();
        $result['exitcode']  = $exitcode;
        $result['message']   = $message;
	
        return $result;
    }
    
     /**
     * Validation routine for sdn_username
     *
     * @param string $sdn_username SDN Username
     *
     * @return boolean TRUE if sdn_username is valid
     */

    function validate_username($username)
    {
        clearos_profile(__METHOD__, __LINE__);

        if (! preg_match("/^[A-Za-z0-9]+$/", $username))
            return lang('base_username_invalid');

        if ((strlen($username) < 4)&&($username!='all'))
            return lang('base_username_too_short') . ' (4).';
    }

    /**
     * Validation network address
     *
     * @param string $address
     *
     * @return boolean TRUE network address is valid
     */

    function validate_address($address)
    {
        clearos_profile(__METHOD__, __LINE__);
	
	$network_utils = new Network_Utils();
	if ((!$network_utils->is_valid_ip($address))&&(!$network_utils->is_valid_network($address)))
	{
	    return lang('network_ip_invalid');
	}
    }
    
        /**
     * Validation password
     *
     * @param string $password
     *
     * @return boolean TRUE password is valid
     */
    
    function validate_password($password)
    {
        clearos_profile(__METHOD__, __LINE__);

        if (! preg_match("/^[A-Za-z0-9\\!\\@\\#\\$\\%\\^\\*\\(\\)\\-\\_\\&]+$/", $password))
            return lang('base_password_is_invalid');

        if (strlen($password) < 4)
            return lang('base_password_too_short') . ' (4).';
    }
    
     /**
     * Validates a port number.
     *
     * @param string $port port number
     *
     * @return string error message if port is invalid
     */

    function validate_port($port)
    {
        clearos_profile(__METHOD__, __LINE__);

        if (! preg_match("/^\d+$/", $port))
            return lang('network_port_invalid');

        if (($port > 65535) || ($port <= 0))
            return lang('network_port_invalid');
    }
    
    /**
     * Validates an IP address.
     *
     * @param string $ip IP address
     *
     * @return string error message if IP is invalid
     */
    
    public static function validate_ip($ip)
    {
        clearos_profile(__METHOD__, __LINE__);

        if (@inet_pton($ip) === FALSE)
            return lang('network_ip_invalid');
    }
    
}

