<?php

/**
 * Server 1C class.
 *
 * @category   apps
 * @package    server-1c
 * @subpackage controllers
 * @author     High Tech Lider ltd. <htlead@htlead.kz>
 * @copyright  2018 High Tech Lider
 * @license    http://www.gnu.org/copyleft/gpl.html GNU General Public License version 3 or later
 * @link       https://github.com/htlead/clear_os_1c_server
 */

///////////////////////////////////////////////////////////////////////////////
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// N A M E S P A C E
///////////////////////////////////////////////////////////////////////////////

namespace clearos\apps\server_1c;

///////////////////////////////////////////////////////////////////////////////
// B O O T S T R A P
///////////////////////////////////////////////////////////////////////////////

$bootstrap = getenv('CLEAROS_BOOTSTRAP') ? getenv('CLEAROS_BOOTSTRAP') : '/usr/clearos/framework/shared';
require_once $bootstrap . '/bootstrap.php';

///////////////////////////////////////////////////////////////////////////////
// T R A N S L A T I O N S
///////////////////////////////////////////////////////////////////////////////

clearos_load_language('1c_server');

///////////////////////////////////////////////////////////////////////////////
// D E P E N D E N C I E S
///////////////////////////////////////////////////////////////////////////////

// Classes
//--------

use \clearos\apps\base\File as File;
use \clearos\apps\base\Folder as Folder;
use \clearos\apps\base\Software as Software;
use \clearos\apps\base\Daemon as Daemon;
use \clearos\apps\base\Shell as Shell;

clearos_load_library('base/File');
clearos_load_library('base/Folder');
clearos_load_library('base/Software');
clearos_load_library('base/Daemon');
clearos_load_library('base/Shell');

// Exceptions
//-----------

use \clearos\apps\base\Validation_Exception as Validation_Exception;

clearos_load_library('base/Validation_Exception');

///////////////////////////////////////////////////////////////////////////////
// C L A S S
///////////////////////////////////////////////////////////////////////////////

/**
 * Server 1C webapps controller.
 *
 * @category   apps
 * @package    server-1c
 * @subpackage controllers
 * @author     High Tech Lider ltd. <htlead@htlead.kz>
 * @copyright  2018 High Tech Lider
 * @license    http://www.gnu.org/copyleft/gpl.html GNU General Public License version 3 or later
 * @link       https://github.com/htlead/clear_os_1c_server
 */

class Server_1C extends Daemon
{
    ///////////////////////////////////////////////////////////////////////////////
    // C O N S T A N T S
    ///////////////////////////////////////////////////////////////////////////////
    
    const HTTP_CONFIG       = '/etc/httpd/conf.d/server_1c.conf';
    const PATH_TO_1C        = '/opt/1C/v8.3/x86_64/';
    const FILE_CONFIG       = '/etc/clearos/server_1c.conf';
    const FILE_LOG          = '/var/log/srv1cv83';
    const PATH_TO_LIC_TOOLS = '/opt/1C/1CE/x86_64/';
    const PATH_TO_LIC       = '/var/1C/licenses';
    const JAVA_HOME         = '/usr/lib/jvm/jre-1.8.0';
        
    private $vhosts      = null;
    private $file_config = null;
    
    ///////////////////////////////////////////////////////////////////////////////
    // M E T H O D S
    ///////////////////////////////////////////////////////////////////////////////

    /**
     * Server 1C constructor.
     */

    public function __construct()
    {
        clearos_profile(__METHOD__, __LINE__);        
        parent::__construct('srv1cv83');
    }
    
    ///////////////////////////////////////////
    ///////////COMMON CONFIG METHODS /////////
    
    public function load_config() {
    	clearos_profile(__METHOD__, __LINE__);
    	if(! is_null($this->file_config)) {
    	    return $this->file_config;
    	}
    	
    	$section = null;
    	$config  = array();
    	
    	$file = new File(Server_1C::FILE_CONFIG, true);
    	if(!$file->exists()) {
		$file->create('root', 'root', '0644');
	}

    	$lines=$file->get_contents_as_array();
	
    	foreach($lines as $line) {
    	
    		$line = trim($line);
    	        if(strlen($line) == 0 || $line[0] == '#') {
    		    continue;
    		}
		$line=preg_replace('/\s+/',' ',$line);
		if(preg_match("/\[([^\]]+)\]/", $line, $m)) {
			$section = $m[1];
			$config[$section] = array();
			continue;
		}
		$line = explode("=", $line, 2);
		$config[$section][trim($line[0])] = trim($line[1]);
    	}

    	$this->file_config = $config;
    	return $config;
    }    
    
    public function save_cfg_config() {
	clearos_profile(__METHOD__, __LINE__);	
	$config_file= array();
	foreach($this->file_config as $section => $data) {
		$config_file[] = "[$section]\r\n";
		if ($section=="logcfg"){
		    $logcfg   = array();
		    $logcfg[] = "<config xmlns=\"http://v8.1c.ru/v8/tech-log\">\r\n";
		    $logcfg[] = "<log location=\"/var/log/srv1cv83\" history=\"168\">\r\n";
		}
		foreach ($data as $key => $value){
		    $config_file[] = "$key = $value\r\n";
		    if (($section=="logcfg")&&($value==1)){
			$logcfg[]="      <event><eq property=\"name\" value=\"$key\"/></event>\r\n";
		    }		    
		}
		if($section=="logcfg"){
		    $logcfg[]="      <property name=\"all\"></property>\r\n";
		    $logcfg[]="</log>\r\n";
		    $logcfg[]="</config>\r\n";
		}
	}
	$file = new File(Server_1C::FILE_CONFIG, true);
	if($file->exists()) {
		$file->delete();
	}
	
	$folder = new Folder(Server_1C::PATH_TO_1C.'conf' , true);
	if(!$folder->exists()){
		$folder->create('root', 'root', '0644');
	}
	    
	$file->create('root', 'root', '0644');
	$file->add_lines(implode('', $config_file));
	
	if(count($logcfg)>0){
	    $file = new File(Server_1C::PATH_TO_1C."conf/logcfg.xml", true);
	    if($file->exists()) {
		    $file->delete();
	    }
	    $file->create('root', 'root', '0644');
	    $file->add_lines(implode('', $logcfg));
	    $folder = new Folder(Server_1C::FILE_LOG, true);
	    if(!$folder->exists()){
		$folder->create('usr1cv8', 'grp1cv8', '0775');
	    }
	}
    } 
    
    public function save_config(){
	$file = new File(Server_1C::FILE_CONFIG, true);
	$config_file=array();
	foreach($this->file_config as $section => $data) {
		$config_file[] = "[$section]\r\n";
		foreach ($data as $key => $value){
		    $config_file[] = "$key = $value\r\n";
		}    
	}
	if($file->exists()) {
		$file->delete();
	}
	$file->create('root', 'root', '0644');
	$file->add_lines(implode('', $config_file));
    }   
    
    public function update_cfg_log($data){
	clearos_profile(__METHOD__, __LINE__);
	$this->load_config();
	foreach($data as $key => $value) {
	    $this->file_config['logcfg'][$key]=$value;
	}
	
	$this->save_cfg_config();
    }
    
    public function get_config_section($section){
	clearos_profile(__METHOD__, __LINE__);
	$this->load_config();	
	$out=$this->file_config[$section];
	return $out;
    }
    
    public function its_subs_add($user, $pass){
	clearos_profile(__METHOD__, __LINE__);
	$this->load_config();	
        $this->file_config['ITS']['username']=$user;
        $this->file_config['ITS']['password']=$pass;
        $this->save_config();
    }
    
    
        /**
     * Validation routine for sdn_username
     *
     * @param string $sdn_username SDN Username
     *
     * @return boolean TRUE if sdn_username is valid
     */

    public function validate_username($username)
    {
        clearos_profile(__METHOD__, __LINE__);

        if (! preg_match("/^[A-Za-z0-9_]+$/", $username))
            return lang('base_username_invalid');

        if (strlen($username) < 4)
            return lang('base_username_too_short') . ' (4).';
    }

    /**
     * Validation routine for sdn_password
     *
     * @param string $sdn_password max instances
     *
     * @return boolean TRUE if sdn_password is valid
     */

    public function validate_password($password)
    {
        clearos_profile(__METHOD__, __LINE__);

        if (! preg_match("/^[A-Za-z0-9\\!\\@\\#\\$\\%\\^\\*\\(\\)\\-\\_\\&]+$/", $password))
            return lang('base_password_is_invalid');

        if (strlen($password) < 4)
            return lang('base_password_too_short') . ' (4).';
    }
    /////////////////////////////////////////
    //////WEB CLIENT PUBLISHING UTIL////////
    
    public function web_get_vhosts() {
	clearos_profile(__METHOD__, __LINE__);
	
	$this->web_load();
	$out = $this->vhosts;
	return $out;
    } 
    
    public function get_vhost($vhost) {
    	clearos_profile(__METHOD__, __LINE__);
	$this->web_load();
    	$data = $this->vhosts[$vhost];
    	return $data;
    }
    
    public function web_load() {
    	clearos_profile(__METHOD__, __LINE__);
    	
    	if(! is_null($this->vhosts)) {
    	    return $this->vhosts;
    	}
    	$section = null;
    	$config  = array();
    	$aliases = array();
    	
    	$file = new File(Server_1C::HTTP_CONFIG, true);
    	if(!$file->exists()) {
		$file->create('root', 'root', '0644');
		$out = array();
		$out[]= "# --------------------------------------------------------------#\r\n" .
			"# This file is generated by Server 1C ClearOS module.	   #\r\n" .
			"# Any change in this file will be lost.			 #\r\n" .
			"# --------------------------------------------------------------#\r\n\r\n";
		$file->add_lines(implode('', $out));
		
	}

    	$lines=$file->get_contents_as_array();
	
    	foreach($lines as $line) {
    	
    		$line = trim($line);    		
    	        if(strlen($line) == 0 || $line[0] == '#') {
    		    continue;
    		}
		$line=preg_replace('/\s+/',' ',$line);
		
		preg_match('/(<Directory)\s+(?P<value>".*?\s*")(\s*>)/', $line, $matches0);
    		    if (isset($matches0['value'])) {
    			foreach($aliases as $alias => $path)
    			{
    		    	    if($path==str_replace('"','',$matches0['value'])){
    		    		$section                  = str_replace('/','',str_replace('"','',$alias));
    		    		$config[$section]['Path'] = str_replace('"','',$path);
    		    		$config[$section]['Alias']= str_replace('/','',str_replace('"','',$alias));
    		    	    } 
    			}
		    }
    		preg_match('/^(?P<key>\w+)\s+(?P<value>.*)/', $line, $matches1);
    		    if (isset($matches1['key'])) {
    			    if($matches1['key']=='Alias'){
    				$alias=explode(' ', $matches1['value']);
    				$aliases[str_replace('"','',$alias[0])]=str_replace('"','',$alias[1]);
    		            }elseif($section!=null){
    		        	$config[$section][$matches1['key']] = $matches1['value'];
    		        	if($matches1['key']=='ManagedApplicationDescriptor'){
    		        		$filename=str_replace('"','',$matches1['value']);
					$file   = new File($filename, TRUE);
					$xmlstr = $file->get_contents();
    					$xmlstr = preg_replace('/\s+/',' ',$xmlstr);
    					preg_match('/(\s*Srvr\s*)(=\s*)(?P<Srvr>.*)(;Ref)/',$xmlstr,$xml_matches);
    					$config[$section]['Srvr']=$xml_matches['Srvr'];
    					preg_match('/(\s*Ref\s*)(=\s*)(?P<Base>.*)(;">)/',$xmlstr,$xml_matches);
    					$config[$section]['Base']=$xml_matches['Base'];
    					
    				}
    		    	    }
    		    }
    		if(preg_match('/<\s*Directory\s*>)/')){ $section=null; }
    	}
    	

    	$this->vhosts = $config;
    	return $config;
    }
    
    public function web_save() {
	clearos_profile(__METHOD__, __LINE__);	
	$out = array();
	$out[]= "# --------------------------------------------------------------#\r\n" .
		"# This file is generated by Server 1C ClearOS module.	   #\r\n" .
		"# Any change in this file will be lost.			 #\r\n" .
		"# --------------------------------------------------------------#\r\n\r\n";
	$out[] = "LoadModule _1cws_module \"".Server_1C::PATH_TO_1C."wsap24.so\"\r\n\r\n";
	ksort($this->vhosts);

	foreach($this->vhosts as $vhost => $data) {
		$out[] = "#1c publication\r\n";
		$out[] = "Alias \"".$vhost."\" \"".$data["Path"]."\"\r\n";
		$out[] = "<Directory \"".$data["Path"]."\">\r\n";
		foreach($data as $key => $value) {
		    if($key=="Srvr" || $key=="Base" || $key=="Path" || $key=="Alias")continue; 
		    $out[] = $key." ".$value."\r\n";
		}
		$out[] = "</Directory>\r\n";
		$out[] = PHP_EOL;
	}

	$file = new File(Server_1C::HTTP_CONFIG, true);
	if($file->exists()) {
		$file->delete();
	}
	$file->create('root', 'root', '0644');
	$file->add_lines(implode('', $out));
    }
    
    public function web_drop($vhost) {
	clearos_profile(__METHOD__, __LINE__);

	$this->web_load();
	$folder = new Folder($this->vhosts[$vhost]['Path'], true);
	if($folder->exists()){
	    $folder->delete(TRUE);
	}
	unset($this->vhosts[$vhost]);
	$this->web_save();
    }
    
    public function web_change_state($vhost){
	clearos_profile(__METHOD__, __LINE__);
	$this->web_load();	
	if($this->vhosts[$vhost]['Require']=='all granted'){
	    $this->vhosts[$vhost]['Require']='all denied';
	}else{
	    $this->vhosts[$vhost]['Require']='all granted';
	}
	$this->web_save();
    }
    
    public function web_update($data){
	clearos_profile(__METHOD__, __LINE__);
	$this->web_load();	
        $this->vhosts[$data['alias']]['SSLVerifyClient']=$data['sslverifyclient'];	
        $this->web_save();
    }
    
    public function web_toogle($vhost, $enable) {
    	clearos_profile(__METHOD__, __LINE__);
	$this->web_load();
    	$this->vhosts[$vhost][Server_1C::ENABLED] = $enable;
    	$this->web_save();
    } 
    
    public function web_publish($data) {
	clearos_profile(__METHOD__, __LINE__);

	$args = sprintf('-apache24 -wsdir %s -dir /var/www/html/%s -connstr "Srvr=%s;Ref=%s;" -confPath /etc/httpd/conf.d/server_1c.conf',$data['alias'],$data['alias'],$data['server'],$data['base']);

        $shell = new Shell();
        
        $exitcode = $shell->execute(Server_1C::PATH_TO_1C.'webinst' , $args, TRUE);
        
        return $exitcode;
    }    
    
    /////////////////////////////////////////
    ////////VALIDATE METHODS/////////////////
    
    public function validate_alias($alias) {
	clearos_profile(__METHOD__, __LINE__);
	if(! preg_match('/^[A-Za-z0-9_]+$/', $alias, $match))
	    return lang('server_1c_web_fail_alias');
	if($add) {
	    $this->load();
	    if(! is_null($this->vhosts[$alias]))
		return lang('server_fail_web_alias_exist');
	}
    }
    
    public function validate_server($server) {
	clearos_profile(__METHOD__, __LINE__);
	if((! preg_match('/^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$/', $server, $match))&&
	    (! preg_match('/^[A-Za-z0-9_]+$/', $server, $match))&&
	    (! preg_match('/^(([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]*[a-zA-Z0-9])\.){2,}([A-Za-z0-9]|[A-Za-z0-9][A-Za-z0-9\-]*[A-Za-z0-9]){2,}$/', $server, $match)))
		return lang('server_1c_web_fail_server');
    }
    
    public function validate_base($base) {
	clearos_profile(__METHOD__, __LINE__);
	if(! preg_match('/^[A-Za-z0-9_]+$/', $base, $match))
	    return lang('server_1c_web_fail_base');
    }
    
    
    public function validate_port($port) {
	clearos_profile(__METHOD__, __LINE__);
	if(! preg_match('/([1-9][0-9]{0,3}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])/', $port, $match))
	    return lang('server_1c_port_val_fail');
    }
    
    
    public function validate_number($number) {
	clearos_profile(__METHOD__, __LINE__);
	if(! preg_match('/^\d+$/', $number, $match))
	    return lang('server_1c_number_val_fail');
    }
    
    
    public function validate_port_range($port_range) {
	clearos_profile(__METHOD__, __LINE__);
	if(! preg_match('/^0*([1-9]|[1-8][0-9]|9[0-9]|[1-8][0-9]{2}|9[0-8][0-9]|99[0-9]|[1-8][0-9]{3}|9[0-8][0-9]{2}|99[0-8][0-9]|999[0-9]|[1-5][0-9]'.
	'{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5]):([1-9]|[1-8][0-9]|9[0-9]|[1-8][0-9]{2}|9[0-8][0-9]|99[0-9]|[1-8][0-9]{3}|9[0-8][0-9]'.
	'{2}|99[0-8][0-9]|999[0-9]|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])$/', $port_range, $match))
	    return lang('server_1c_port_range_val_fail');
    }
    
    
    
    
    
     /* Return 1C version
     *
     * @ return string
     */
     
     public function get_info()
     {
        clearos_profile(__METHOD__, __LINE__);        
        
        $pkglist     = array('1C_Enterprise83-common', '1C_Enterprise83-common-nls', '1C_Enterprise83-server', '1C_Enterprise83-server-nls', '1C_Enterprise83-ws', '1C_Enterprise83-ws-nls', ' 1c-enterprise-license-tools', '1c-enterprise-ring');
        
        $return_values= array();
        
        foreach ($pkglist as $pkg)        {
    	    $software= new Software($pkg); 
    	    if ($software->is_installed()){       

    	    $item['package_name']    = $software->get_package_name();
            $item['version'] 	 = $software->get_version();
	    $item['release'] 	 = $software->get_release();
	    $item['install_time'] = $software->get_install_time();
	    $item['install_size'] = $this->formatBytes($software->get_install_size(),2);

	    $return_values[] = $item;
	    }
        }
        return $return_values;
     } 
    
     function formatBytes($size, $precision = 2)
     {
        $base = log($size, 1024);
        $suffixes = array('', 'K', 'M', 'G', 'T');   
            
        return round(pow(1024, $base - floor($base)), $precision) .' '. $suffixes[floor($base)];
     }
     
     function login1c($user, $pass){
	
	$ch = curl_init();
	$url='https://login.1c.ru/login';
	$agent = $_SERVER["HTTP_USER_AGENT"];
	curl_setopt($ch, CURLOPT_USERAGENT, $agent);
	curl_setopt($ch, CURLOPT_URL, $url );
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
	
	$postResult = curl_exec($ch);
	
	$dom = new \DOMDocument;
	$execution = '';
    	$success=$dom->loadHTML($postResult);
    	if($success){
    		$form = $dom -> getElementById('loginForm');
    		if(!empty($form)){
    		    $input_list = $form -> getElementsByTagName('input');
    		    foreach ($input_list as $input)
    		    {
    			if($input->getAttribute('name')=='execution'){
    			    $execution=$input->getAttribute('value');
    			    break;
    			}
    		    }
    		}
    	
    	}

	$params=array( 
    	    'inviteCode'=>'', 
    	    'username' => $user,
    	    'password' => $pass,
    	    'execution' => $execution,
    	    '_eventId' => 'submit',
    	    'geolocation' => '', 
    	    'submit' => 'Войти',
    	    'rememberMe' => 'on');
    	    
    	$url='https://login.1c.ru/login';
 
	$agent = $_SERVER["HTTP_USER_AGENT"];
	curl_setopt($ch, CURLOPT_USERAGENT, $agent);
	curl_setopt($ch, CURLOPT_URL, $url );
	curl_setopt($ch, CURLOPT_POST, 1 );
	curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($params));
	curl_setopt($ch, CURLOPT_COOKIEJAR,  CLEAROS_TEMP_DIR .'/platform_server_1c.txt');
	curl_setopt($ch, CURLOPT_COOKIEFILE,  CLEAROS_TEMP_DIR .'/platform_server_1c.txt');
	
	//Execute the action to login
	$postResult = curl_exec($ch);
	return $ch;
     }
     
     function get_rpm_list()
     {
        $result=array();
        
        try
        {
    	    $its = $this->get_config_section('ITS');
    	    $ch  = $this->login1c($its['username'],$its['password']);
    	    $url = 'https://releases.1c.ru/project/Platform83';
    	    curl_setopt($ch, CURLOPT_URL, $url);
    	    $postResult = curl_exec($ch);
         
    	    $dom = new \DOMDocument;
    	    $success=$dom->loadHTML($postResult);
    	    if($success){
    		$table = $dom -> getElementById('versionsTable');
    		if(!empty($table)){
    		    $tr_list = $table -> getElementsByTagName('tr');
    		    $items = array();
    		    foreach ($tr_list as $tr)
    		    {
    			$td_list = $tr -> getElementsByTagName('td');
    			foreach ($td_list as $td){
			    foreach ($td -> getElementsByTagName('a') as $a)
			    {
				$item=explode('.',trim($a->nodeValue));
				$items[]=$item;
			    }
			}
    		    }
    		    $result['exitcode'] = 0;
    		    $result['items']    = $items;
    		}else
    		{
    		    $result['exitcode'] = 1;
    		    $result['items']    = array();
    		    $result['errmsg']   = lang('server_1c_rpmlist_load_error');
    		}
    	    }else
    	    {
    		$result['exitcode'] = 1;
    		$result['items']    = array();
    		$result['errmsg']   = lang('server_1c_rpmlist_load_error');	
    	    }
    	}catch(Exception $e)
    	{
    	    $result['exitcode'] = 1;
    	    $result['items']    = array();
    	    $result['errmsg']   = $e->getMessage();
    	}
                
        return $result;
     }
     
     function install_rpm($platform)
     {
        $result='';
	$shell = new Shell();
	$options = array('background' => TRUE);
	
	
	try{            
            $args = sprintf('%s %s','/usr/clearos/apps/server_1c/htdocs/platform_download.php', $platform);
    	    $exitcode = $shell->execute('/usr/bin/php' , $args, TRUE, $options);
    	    $result.="</BR></BR>".implode("</BR>",$shell->get_output());
        }catch(Exception $e)
        {
    	    $result.=$e->getMessage();
        }
        
        return $result;
     }
     
     function install_platform()
     {
        clearos_profile(__METHOD__, __LINE__);
	
	$output='';
	
	$shell = new Shell();
	$options = array();
	$options['validate_exit_code']=FALSE;
	
	try{
	    $args = sprintf(' %s', CLEAROS_TEMP_DIR.'/1C_Enterprise83-*.rpm');
	
    	    $exitcode = $shell->execute('rm' , $args, TRUE, $options);
    	    $output.="</BR>".implode("</BR>",$shell->get_output());
            
            $args = sprintf('-zxf %s -C %s',CLEAROS_TEMP_DIR .'/1c.tar.gz', CLEAROS_TEMP_DIR);
    	    $exitcode = $shell->execute('tar' , $args, TRUE, $options);
    	    $output.="</BR>".implode("</BR>",$shell->get_output());
    	    
    	    $args = sprintf('-i %s', CLEAROS_TEMP_DIR.'/1C_Enterprise83-*.rpm');
	
    	    $exitcode = $shell->execute('rpm' , $args, TRUE, $options);
    	    $output.="</BR>".implode("</BR>",$shell->get_output());
    	        	    
        }catch(Exception $e)
        {
    	    $output.=$e;
        }
        

        $result=array();
        $result['exitcode'] = $exitcode;
        $result['output']   = $output;
        
        return $result;
     }
     
     function uninstall_platform()
     {
        clearos_profile(__METHOD__, __LINE__);
	
	$output='';
	
	$shell = new Shell();
	$options = array();
	$options['validate_exit_code']=FALSE;
	
	try{

    	    $args = sprintf('-y remove %s %s %s %s %s %s %s %s', '1C_Enterprise83-common', '1C_Enterprise83-common-nls', '1C_Enterprise83-server', '1C_Enterprise83-server-nls', '1C_Enterprise83-ws', '1C_Enterprise83-ws-nls', ' 1c-enterprise-license-tools', '1c-enterprise-ring');
	
    	    $exitcode = $shell->execute('yum' , $args, TRUE, $options);
    	    $output.="</BR></BR>".implode("</BR>",$shell->get_output());

        }catch(Exception $e)
        {
    	    $output.=$e;
        }
        

        $result=array();
        $result['exitcode'] = $exitcode;
        $result['output']   = $output;
        
        return $result;
     } 
     
     function uninstall_package($package)
     {
        clearos_profile(__METHOD__, __LINE__);
	
	$output='';
	
	$shell = new Shell();
	$options = array();
	$options['validate_exit_code']=FALSE;
	
	try{

    	    $args = sprintf('-y remove %s ', $package);
    	    $exitcode = $shell->execute('yum' , $args, TRUE, $options);
    	    $output.="</BR></BR>".implode("</BR>",$shell->get_output());

        }catch(Exception $e)
        {
    	    $output.=$e;
        }
        

        $result=array();
        $result['exitcode'] = $exitcode;
        $result['output']   = $output;
        
        return $result;
     }
    
    function pg_install() {
    	clearos_profile(__METHOD__, __LINE__);
	
	$result='';
	
	$shell = new Shell();
	$options = array();
	$options['validate_exit_code']=FALSE;
	
	try{
            $args = sprintf('-ivh %s','http://1c.postgrespro.ru/keys/postgrespro-1c-centos96.noarch.rpm');
    	    $exitcode = $shell->execute('rpm' , $args, TRUE, $options);
    	    $result.="</BR></BR>".implode("</BR>",$shell->get_output());

    	    $args = sprintf('makecache');
	
    	    $exitcode = $shell->execute('yum' , $args, TRUE, $options);
    	    $result.="</BR></BR>".implode("</BR>",$shell->get_output());
    	    
    	    $args = sprintf('install -y %s','postgresql-pro-1c-9.6');
	
    	    $exitcode = $shell->execute('yum' , $args, TRUE, $options);
    	    $result.="</BR></BR>".implode("</BR>",$shell->get_output());
        }catch(Exception $e)
        {
    	    $result.=$e;
        }
        
        return $result;
    }
    
    function ring_install()
    {
	clearos_profile(__METHOD__, __LINE__);
	
	$shell = new Shell();
	$options = array();
	$options['validate_exit_code']=FALSE;
	
	$args = sprintf('-i %s','/opt/1C/v8.3/x86_64/ExtDst/*.rpm');

    	$exitcode = $shell->execute('rpm' , $args, TRUE, $options);

        return implode("</BR>",$shell->get_output());
    }
    
    /*Functions for work with
      Licenses
    */
    
     function pg_initdb()
    {
	clearos_profile(__METHOD__, __LINE__);
	
	$shell = new Shell();
	$options = array();
	$options['validate_exit_code']=FALSE;
	
	$args = sprintf('postgresql-9.6 initdb');

        $exitcode = $shell->execute('service' , $args, TRUE, $options);

        return implode("</BR>",$shell->get_output());
    }
    
    
     function java_install() {
    	clearos_profile(__METHOD__, __LINE__);
	
	$result='';
	
	$shell = new Shell();
	$options = array();
	$options['validate_exit_code']=FALSE;
	
	try{
            $args = sprintf('install %s','java');
    	    $exitcode = $shell->execute('yum' , $args, TRUE, $options);
    	    $result.="</BR></BR>".implode("</BR>",$shell->get_output());

	    $file = new File('/etc/environment',TRUE);
	    
	    $lines   = $file->get_contents_as_array();
	
  $content = array();
    	    foreach($lines as $line) {
    	
    		$line = trim($line);
    	        if(strlen($line) == 0 || $line[0] == '#') {
    		    continue;
    		}

		$line = explode("=", $line, 2);
		$content[trim($line[0])] = trim($line[1]);
    	    }
    	    
	    $content['JAVA_HOME']='/usr/lib/jvm/jre-1.8.0-openjdk';
	    $config_file=array();
            foreach ($content as $key => $value){
		    $config_file[] = "$key=\"$value\"\n";
		    }    
	    if($file->exists()) {
		$file->delete();
	     }
	     $file->create('root', 'root', '0644');
	     $file->add_lines(implode('', $config_file));
	    
        }catch(Exception $e)
        {
    	    $result.=$e;
        }
        
        return $result;
     }
    
     public function get_licenses() {
	clearos_profile(__METHOD__, __LINE__);
	
	$shell = new Shell();
	
	$args = sprintf('license list --path %s',Server_1C::PATH_TO_LIC);

        $exitcode = $shell->execute(Server_1C::PATH_TO_LIC_TOOLS.'ring/ring' , $args, TRUE);

        return $shell->get_output();
    }
    
     public function get_license_info($license) {
	clearos_profile(__METHOD__, __LINE__);
	
	$shell = new Shell();
	
	$args = sprintf('license info --name %s',$license);

        $exitcode = $shell->execute(Server_1C::PATH_TO_LIC_TOOLS.'ring/ring' , $args, TRUE);

        return $shell->get_output();
    }
    
    public function set_license_work_places($work_places, $license){
    	clearos_profile(__METHOD__, __LINE__);
	$this->load_config();
        $this->file_config['licenses'][$license]=$work_places;

	$this->save_lic_config();
    }    
    
    public function get_license_work_places($license){
    	clearos_profile(__METHOD__, __LINE__);
	$this->load_config();
        $out=$this->file_config['licenses'][$license];

	return $out;
    }
    
    /*Functions for work with
      Remote Administrative Client Utility
    */
    
    function get_clusters_list() {
	clearos_profile(__METHOD__, __LINE__);
	
	$shell = new Shell();
	
	$args = sprintf('cluster list');

        $exitcode = $shell->execute(Server_1C::PATH_TO_1C.'rac' , $args, TRUE);
        
        return $shell->get_output();
    }
    
    function get_cluster_info($cluster) {
	clearos_profile(__METHOD__, __LINE__);
	
	$shell = new Shell();
	
	$args = sprintf('cluster info --cluster=%s',$cluster);

        $exitcode = $shell->execute(Server_1C::PATH_TO_1C.'rac' , $args, TRUE);
        
        return $shell->get_output();
    }
    
    function add_cluster($host, $port, $name, $expiration_timeout, $lifetime_limit, $max_memory_size, $max_memory_time_limit, $security_level, 
    					$session_fault_tolerance_level, $load_balancing_mode, $errors_count_threshold, $kill_problem_processes) {
	clearos_profile(__METHOD__, __LINE__);
	
	$shell = new Shell();
	
	$options = array();
	$options['validate_exit_code']=FALSE;
		
	$args = sprintf('cluster  localhost:1545 insert --host=%s --port=%s  --name=%s --expiration-timeout=%s --lifetime-limit=%s --max-memory-size=%s  --max-memory-time-limit=%s'.
	                 ' --security-level=%s --session-fault-tolerance-level=%s --load-balancing-mode=%s --errors-count-threshold=%s --kill-problem-processes=%s',
	                 $host, $port, $name, $expiration_timeout, $lifetime_limit, $max_memory_size, $max_memory_time_limit, $security_level, 
	                 $session_fault_tolerance_level, $load_balancing_mode, $errors_count_threshold, $kill_problem_processes );
        

        $exitcode = $shell->execute(Server_1C::PATH_TO_1C.'rac' , $args, TRUE, $options);
        
        
        $result=array();
        $result['exitcode'] = $exitcode;
        $result['output']   = implode('</BR>',$shell->get_output());
        
        return $result;
    }
    function update_cluster($cluster, $name, $expiration_timeout, $lifetime_limit, $max_memory_size, $max_memory_time_limit, $security_level, 
    					$session_fault_tolerance_level, $load_balancing_mode, $errors_count_threshold, $kill_problem_processes) {
	clearos_profile(__METHOD__, __LINE__);
	
	$shell = new Shell();
	
	$options = array();
	$options['validate_exit_code']=FALSE;
		
	$args = sprintf('cluster  update --cluster=%s  --name=%s --expiration-timeout=%s --lifetime-limit=%s --max-memory-size=%s  --max-memory-time-limit=%s'.
	                 ' --security-level=%s --session-fault-tolerance-level=%s --load-balancing-mode=%s --errors-count-threshold=%s --kill-problem-processes=%s',
	                 $cluster, $name, $expiration_timeout, $lifetime_limit, $max_memory_size, $max_memory_time_limit, $security_level, 
	                 $session_fault_tolerance_level, $load_balancing_mode, $errors_count_threshold, $kill_problem_processes );
        
        $exitcode = $shell->execute(Server_1C::PATH_TO_1C.'rac' , $args, TRUE, $options);
        
        
        $result=array();
        $result['exitcode'] = $exitcode;
        $result['output']   = implode('</BR>',$shell->get_output());
        
        return $result;
    }
    
    function drop_cluster($cluster) {
    	clearos_profile(__METHOD__, __LINE__);
	
	$shell = new Shell();
	
	$args = sprintf('cluster remove --cluster=%s',$cluster);
	
        $exitcode = $shell->execute(Server_1C::PATH_TO_1C.'rac' , $args, TRUE);
        
        return $shell->get_output();
    }
    
    function get_servers_list($cluster) {
	clearos_profile(__METHOD__, __LINE__);
	
	$shell = new Shell();
	
	$args = sprintf('server list --cluster=%s',$cluster);

        $exitcode = $shell->execute(Server_1C::PATH_TO_1C.'rac' , $args, TRUE);
        
        return $shell->get_output();
    } 
    
    function get_server_info($cluster, $server) {
	clearos_profile(__METHOD__, __LINE__);
	
	$shell = new Shell();
	
	$args = sprintf('server --cluster=%s info --server=%s',$cluster, $server);

        $exitcode = $shell->execute(Server_1C::PATH_TO_1C.'rac' , $args, TRUE);
        
        return $shell->get_output();
    } 
    
    function add_server($cluster,$agent_host, $agent_port, $port_range, $name, $using, $infobases_limit, $memory_limit, $connections_limit, $cluster_port, 
			$dedicate_managers, $sw_processes_memory_limit, $safe_call_memory_limit) {
	clearos_profile(__METHOD__, __LINE__);
	
	$shell = new Shell();
	
	$options = array();
	$options['validate_exit_code']=FALSE;
		
	$args = sprintf('server --cluster=%s insert --agent-host=%s --agent-port=%s --port-range=%s --name=%s --using==%s --infobases-limit=%s --memory-limit=%s'. 
	                ' --connections-limit=%s --cluster-port=%s --dedicate-managers=%s --safe-working-processes-memory-limit=%s --safe-call-memory-limit=%s',
	                 $cluster, $agent_host, $agent_port, $port_range, $name, $using, $infobases_limit, $memory_limit, $connections_limit, $cluster_port, 
	                 $dedicate_managers, $sw_processes_memory_limit, $safe_call_memory_limit );
        
        $exitcode = $shell->execute(Server_1C::PATH_TO_1C.'rac' , $args, TRUE, $options);
        
        
        $result=array();
        $result['exitcode'] = $exitcode;
        $result['output']   = implode('</BR>',$shell->get_output());
        
        return $result;
    }
    
    
    function update_server($cluster, $server, $port_range, $using, $infobases_limit, $memory_limit, $connections_limit,
			$dedicate_managers, $sw_processes_memory_limit, $safe_call_memory_limit) {
	clearos_profile(__METHOD__, __LINE__);
	
	$shell = new Shell();
	
	$options = array();
	$options['validate_exit_code']=FALSE;
		
	$args = sprintf('server update --cluster=%s --server=%s --port-range=%s --using==%s --infobases-limit=%s --memory-limit=%s'. 
	                ' --connections-limit=%s --dedicate-managers=%s --safe-working-processes-memory-limit=%s --safe-call-memory-limit=%s',
	                 $cluster, $server, $port_range, $using, $infobases_limit, $memory_limit, $connections_limit,
	                 $dedicate_managers, $sw_processes_memory_limit, $safe_call_memory_limit );
        

        $exitcode = $shell->execute(Server_1C::PATH_TO_1C.'rac' , $args, TRUE, $options);
        
        
        $result=array();
        $result['exitcode'] = $exitcode;
        $result['output']   = implode('</BR>',$shell->get_output());
        
        return $result;
    }
    
    function drop_server($cluster, $server) {
    	clearos_profile(__METHOD__, __LINE__);
	
	$shell = new Shell();
	
	$args = sprintf('server --cluster=%s remove --server=%s',$cluster, $server);
	
        $exitcode = $shell->execute(Server_1C::PATH_TO_1C.'rac' , $args, TRUE);
        
        return $shell->get_output();
    }
    
    function get_infobases_list($cluster) {
	clearos_profile(__METHOD__, __LINE__);
	
	$shell = new Shell();
	
	$args = sprintf('infobase summary list --cluster=%s',$cluster);

        $exitcode = $shell->execute(Server_1C::PATH_TO_1C.'rac' , $args, TRUE);
        
        return $shell->get_output();
    }    
    
    function get_sessions_list($cluster, $infobase) {
	clearos_profile(__METHOD__, __LINE__);
	
	$shell = new Shell();
	
	$args = sprintf('session list --cluster=%s --infobase=%s',$cluster, $infobase);

        $exitcode = $shell->execute(Server_1C::PATH_TO_1C.'rac' , $args, TRUE);
        
        return $shell->get_output();
    }
    
    function terminate_session($cluster, $session) {
	clearos_profile(__METHOD__, __LINE__);
	
	$shell = new Shell();
	
	$args = sprintf('session terminate --cluster=%s --session=%s',$cluster, $session);

        $exitcode = $shell->execute(Server_1C::PATH_TO_1C.'rac' , $args, TRUE);
        
        return $shell->get_output();
    }
    
    function add_infobase($cluster, $name, $dbms, $db_server, $db_name, $db_user, $db_pwd, $create_database, $locale, $scheduled_jobs_deny) {
	clearos_profile(__METHOD__, __LINE__);
	
		
	$shell = new Shell();
	
	$options = array();
	$options['validate_exit_code']=FALSE;
	
	$create_db_param = ($create_database) ? '--create-database' : '';
	$date_offset     = ($dbms=='MSSQLServer') ? 2000 : 0;
	
	$args = sprintf('infobase --cluster=%s create %s --name=%s --dbms=%s --db-server=%s --db-name=%s --db-user=%s --db-pwd=%s --locale=%s --scheduled-jobs-deny=%s --date-offset=%s   --license-distribution=allow',
	                 $cluster, $create_db_param, $name , $dbms, $db_server, $db_name, $db_user, $db_pwd , $locale, $scheduled_jobs_deny, $date_offset);
        
        $exitcode = $shell->execute(Server_1C::PATH_TO_1C.'rac' , $args, TRUE, $options);
        
        
        $result=array();
        $result['exitcode'] = $exitcode;
        $result['output']   = implode('</BR>',$shell->get_output());
        
        return $result;
    }
    
    function drop_infobase($cluster, $infobase) {
    	clearos_profile(__METHOD__, __LINE__);
	
	$shell = new Shell();
	
	$args = sprintf('infobase --cluster=%s drop --infobase=%s --drop-database',$cluster, $infobase);
	
        $exitcode = $shell->execute(Server_1C::PATH_TO_1C.'rac' , $args, TRUE);
        
        return $shell->get_output();
    }
}
